use std::process;

use log::{error, info};

mod config;
mod error;
mod feature;
mod negotiator;
mod server;
mod streamstart;
mod tag;
mod tls;

#[tokio::main]
async fn main() -> Result<(), anyhow::Error> {
    if !std::env::var("LOG").is_ok() {
        #[cfg(debug_assertions)]
        std::env::set_var("LOG", "debug");
        #[cfg(not(debug_assertions))]
        std::env::set_var("LOG", "info");
    }
    pretty_env_logger::init_custom_env("LOG");

    let cfg = match config::Config::load() {
        Ok(cfg) => cfg,
        Err(err) => {
            error!(
                "getting config: {err}. writing default to {}",
                config::DEFAULT_PATH
            );
            config::Config::default().save(config::DEFAULT_PATH)?;
            process::exit(1);
        }
    };
    info!("checking for certificates");
    let certs = cfg.certificate().await.expect("getting certificates");

    let host = cfg.hostname();
    info!("listening on {host}:{}!", cfg.port);
    server::listen(host, cfg.port).await.unwrap();
    Ok(())
}
