use async_trait::async_trait;
use quick_xml::{
    events::{BytesStart, Event},
    Writer,
};
use tokio::io::{AsyncBufRead, AsyncWrite};
use tokio_rustls::rustls;

use crate::{
    error::StreamError,
    tag::{self, Tag},
};

pub enum Step {
    Proceed,
    Failure,
}

#[async_trait]
impl Tag for Step {
    async fn write_tag<W>(&self, writer: W) -> Result<(), crate::error::StreamError>
    where
        W: AsyncWrite + Unpin + Send,
    {
        let mut writer = Writer::new(writer);
        writer
            .write_event_async(Event::Empty(
                BytesStart::new(match self {
                    Step::Proceed => "proceed",
                    Step::Failure => "failure",
                })
                .with_attributes(vec![("xmlns", tag::TLS_NAMESPACE)]),
            ))
            .await?;
        Ok(())
    }
}

pub async fn start_tls<R, W>(
    reader: R,
    writer: W,
    start_tls_event: BytesStart<'_>,
) -> Result<Step, StreamError>
where
    R: AsyncBufRead + Unpin,
    W: AsyncWrite + Unpin + Send,
{
    match start_tls_event.try_get_attribute("xmlns") {
        Ok(namespace) => {
            if &namespace
                .map(|a| String::from_utf8(a.value.as_ref().to_vec()).unwrap_or_default())
                .unwrap_or_default()
                != tag::TLS_NAMESPACE
            {
                return Ok(Step::Failure);
            }
        }
        Err(_) => return Ok(Step::Failure),
    }

    // let config = rustls::ServerConfig::builder()
    //     .with_safe_defaults()
    //     .with_no_client_auth()
    //     .with_single_cert(certs, keys.remove(0))
    //     .map_err(|err| io::Error::new(io::ErrorKind::InvalidInput, err))?;

    Step::Proceed.write_tag(writer).await?;

    // match TlsConnector::builder(). {
    //     Ok(conn) => conn.,
    //     Err(err) => {
    //         error!("getting a tls connector: {err}");
    //         return Ok(Step::Failure);
    //     }
    // }

    std::thread::sleep(std::time::Duration::from_secs(3));
    todo!()
}
